def split_on_dot(line: str, only_last_dot=False):
    """
    Split a string on the dot character (.).

    :param line: The line ot split on.
    :param only_last_dot: Only split on the last occurrence of the dot.

    :raises ValueError: if the line does not have a dot.
    """
    if "." not in line:
        # TODO: This is used in the parser for section.keys where a more specific
        # error message would be nice, but it is only used more generically to, say,
        # split a file path. We may just have to insert some try-catches to catch the
        # ValueError and insert a more specific error message or create a custom error
        # for this method?
        raise ValueError(
            "section_key must contain the section and key separated by a dot. "
            + "e.g. 'section.key'"
        )

    return line.rsplit(".", 1) if only_last_dot else line.split(".")



from abc import ABC, abstractmethod


class BaseParser(ABC):
    """
    The abstract base parser for all other parsers to implement.

    All the keys are specified in a 'dot' syntax. So if you have
    sections with subsections with a key that you're trying to get
    the value of, you'd use 'section.subsection.key' as the key parameter.
    """

    @abstractmethod
    def __init__(self, file_contents: str):
        """
        All parsers will take in the file as a string, parse
        them internally to their own representation that they
        can work with, and are able to stringify them back out
        when we need to save (write) them.

        It is up to the caller of the parser to read in the file to
        the parser and save the file when they are done using stringify().
        """
        self.contents = file_contents
        self.parsed_content = self.parse(self.contents)

    @abstractmethod
    def get(self, key, parse_type=True):
        """
        Retrieve the value of a key in its native type.
        This means the string 'true' will be parsed back as the
        boolean True.

        If parse_type is set to False, all values will be returned
        back as strings.
        """
        raise NotImplementedError

    @abstractmethod
    def set(self, key, value):
        """Sets the value of a key."""
        raise NotImplementedError

    @abstractmethod
    def delete(self, section_key):
        """Deletes a key/value pair or entire sections."""
        raise NotImplementedError

    @abstractmethod
    def stringify(self) -> str:
        """
        Using self.__contents, converts the parsed internal representation
        of the file back into a string.
        """
        raise NotImplementedError

    @abstractmethod
    def has(self, section_key: str) -> bool:
        """
        Check if a section, sub-section, or key exists in the file
        using a section.key format.

        Some formats, like JSON, do not have sections and therefore,
        it would only be checking if
        """
        raise NotImplementedError

    @abstractmethod
    def parse(self, file_contents: str):
        """
        Parse the file_contents into an internal representation
        the given parser can work with. It should not return anything
        if you're going to be using the super() constructor.
        """
        raise NotImplementedError


class ParsingError(Exception):
    """Unable to parse the configuration file"""



"""
Parse types allows you to check or parse a string into its native type.

Example:
    parse_value('5') -> 5   (int)
    parse_value('-5') -> -5 (int)

    parse_value('5.52') -> 5.52   (float)
    parse_value('-53.2') -> -53.2 (float)

    parse_value('blahfafda') -> 'blahfafda' (string)

    parse_value('true') -> True (bool)
    parse_value('FALSE') -> False (bool)


"5".isdigit() -> True
"-5".isdigit() -> False
can_be_parsed_as_int("-5") -> True
"""
import re
from distutils.util import strtobool


def parse_value(value):
    if can_be_parsed_as_int(value):
        return int(value)

    if can_be_parsed_as_float(value):
        return float(value)

    if can_be_parsed_as_bool(value):
        return value if type(value) is bool else bool(strtobool(value))

    return value


def can_be_parsed_as_int(value) -> bool:
    if type(value) is int:
        return True

    if type(value) is not str:
        return False

    if value.startswith("-"):
        return value[1:].isdigit()

    return value.isdigit()


def can_be_parsed_as_float(value) -> bool:
    if type(value) is float:
        return True

    if type(value) is not str:
        return False

    if value.startswith("-"):
        return bool(re.match(r"^\d*\.\d+$", value[1:]))

    return bool(re.match(r"^\d*\.\d+$", value))


def can_be_parsed_as_bool(value) -> bool:
    if type(value) is bool:
        return True

    if type(value) is not str:
        return False

    return value.lower() == "true" or value.lower() == "false"



from config_file.config_file import ConfigFile
from config_file.parsers.base_parser import BaseParser, ParsingError

__version__ = "0.4.0"

__all__ = [
    "ConfigFile",
    "ParsingError",
    "BaseParser",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['config_file', 'config_file.parsers']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml>=5.2,<6.0', 'toml>=0.10.0,<0.11.0']

setup_kwargs = {
    'name': 'config-file',
    'version': '0.4.0',
    'description': 'Manage your configuration files.',
    'long_description': '# Config File \n\n> Manage and manipulate your configuration files\n\n[![Python 3.8.0|](https://img.shields.io/badge/python-3.8.0-blue.svg)](https://www.python.org/downloads/release/python-380/) \n[![Version](https://img.shields.io/pypi/v/config-file)](https://pypi.org/project/config-file/)\n[![Style](https://img.shields.io/badge/code%20style-black-000000.svg)](https://pypi.org/project/black/)\n[![Build Status](https://travis-ci.com/eugenetriguba/config_file.svg?branch=master)](https://travis-ci.com/eugenetriguba/config_file)\n\n> This python package is currently a work in progress and is in a pre-alpha phase. The API is liable to break until v1.\n\nConfig File allows you to use the same simple API for manipulating INI, JSON, \nYAML, and TOML configuration files. For the time being, it only supports INI.\n\n## Installation\n```bash\n$ pip install config_file\n```\n\n## Examples\n\n`config.ini`\n```ini\n[calendar]\ntoday = monday\nstart_week_on_sunday = false\ntoday_index = 0\nquarter_hours_passed = 0.25\n```\n\n`example.py`\n```python\nfrom config_file import ConfigFile\n\nconfig = ConfigFile("~/.config/test/config.ini")\n\n# Output your config file as a string\nconfig.stringify()\n>>> \'[calendar]\\ntoday = monday\\nstart_week_on_sunday = false\\ntoday_index = 0\\nquarter_hours_passed = 0.25\\n\\n\'\n\n# Retrieve values with a section.key format\nconfig.get("calendar.today")\n>>> \'monday\'\n\n# Values from the config file are automatically parsed\nconfig.get("calendar.start_week_on_sunday")\n>>> False\n\n# Values from the config file are automatically parsed\nconfig.get("calendar.start_week_on_sunday")\n>>> False\n\n# Unless you don\'t want them to be parsed\nconfig.get("calendar.today_index", parse_type=False)\n>>> \'0\'\n\n# The dot syntax is also used to set values. True is returned on success.\nconfig.set("calendar.today_index", 20)\n>>> True\nconfig.stringify()\n>>> \'[calendar]\\ntoday = monday\\nstart_week_on_sunday = false\\ntoday_index = 20\\nquarter_hours_passed = 0.25\\n\\n\'\n\n# If you specify a section that isn\'t in your config file, the section and the key are added for you.\nconfig.set("week.tuesday_index", 2)\n>>> True\nconfig.stringify()\n>>> \'[calendar]\\ntoday = monday\\nstart_week_on_sunday = false\\ntoday_index = 20\\nquarter_hours_passed = 0.25\\n\\n[week]\\ntuesday_index = 2\\n\\n\'\n\n# Delete can delete an entire section or just a key/value pair.\nconfig.delete(\'week\')\n>>> True\nconfig.stringify()\n>>> \'[calendar]\\ntoday = monday\\nstart_week_on_sunday = false\\ntoday_index = 20\\nquarter_hours_passed = 0.25\\n\\n\'\n\n# Delete can delete an entire section or just a key/value pair.\nconfig.delete(\'calendar.today\')\n>>> True\nconfig.stringify()\n>>> \'[calendar]\\nstart_week_on_sunday = false\\ntoday_index = 20\\nquarter_hours_passed = 0.25\\n\\n\'\n\n# You can also just check if the file has a particular section or key.\nconfig.has(\'calendar\')\n>>> True\nconfig.has(\'week\')\n>>> False\nconfig.has(\'calendar.start_week_on_sunday\')\n>>> True\n\n# The file is only written back out when you call save()\nconfig.save()\n>>> True\n\n# You can also reset the file back to its original state. The current configuration file \n# would be deleted and replaced by a copy of the original. By default, since our passed in\n# config file was at path `~/.config/test/config.ini`, `reset()` will look for \n# `~/.config/test/config.original.ini`\nconfig.reset()\n>>> True\n\n# Or you can specify the file path explicitly\nconfig.reset(original_file_path="~/some_other_directory/this_is_actually_the_original.ini")\n>>> True\n```\n',
    'author': 'Eugene Triguba',
    'author_email': 'eugenetriguba@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/eugenetriguba/config_file',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import configparser
from io import StringIO

from config_file.parsers.base_parser import BaseParser, ParsingError
from config_file.parsers.parse_value import parse_value
from config_file.utils import split_on_dot


class IniParser(BaseParser):
    def __init__(self, file_contents: str):
        """Reads in the file contents into the configparser."""
        super().__init__(file_contents)

    def parse(self, file_contents: str):
        try:
            parser = configparser.ConfigParser()
            parser.read_string(file_contents)
            return parser
        except configparser.Error as error:
            raise ParsingError(error)

    def get(self, section_key, parse_type=True):
        """
        Read the value of `section.key` of the config file.

        :param section_key: The section and key to read from in the config file.
        e.g. 'section1.key2'
        :param parse_type: Coerces the return value to its native type.
        :return: The value of the key, parsed to its native type if parse_type is True.

        :raises ValueError: if the section_key given is not in a dot format. e.g.
                            'section1.key'
        :raises ParsingError: if the specified `section.key` is not found, in an
        invalid format, or if we are unable to coerce the return value to value_type.
        """
        # TODO: Support for retrieving entire sections instead of just key/values
        section, key = split_on_dot(section_key, only_last_dot=True)

        try:
            value = self.parsed_content.get(section, key)
            return parse_value(value) if parse_type else value
        except configparser.Error as error:
            raise ParsingError(error.message)

    def set(self, section_key, value):
        """
        Sets the value of 'section.key' of the config file. If the specified section
        is not in the configuration file, it will be created before adding the key
        to it.

        :param section_key: The key to set from the config file. e.g. 'section1.key'
        :param value: The value to set the key to.

        :return: True if the setting was successful.
        :rtype: boolean

        :raises ValueError: If there is no dot (.) in section_key
        """
        section, key = split_on_dot(section_key, only_last_dot=True)

        if value is not None and not self.parsed_content.has_section(section):
            self.parsed_content.add_section(section)

        if not isinstance(value, str):
            value = str(value)

        self.parsed_content.set(section, key, value)
        return True

    def delete(self, section_key):
        """
        Deletes a key or an entire section.

        :param section_key: The key to delete from the config file.
        e.g. 'ocr.engine'. If no dot (.) is present, it will assume you are trying
        to delete the entire section.

        :return: True if the deletion succeeded.
        :rtype: boolean

        :raise ValueError: If the section or key does not exist in the config file.
        """
        if "." not in section_key:
            if not self.parsed_content.has_section(section_key):
                raise ValueError(
                    f"Cannot delete {section_key} because it is not in the config file."
                )

            self.parsed_content.remove_section(section_key)
            return True

        section, key = split_on_dot(section_key, only_last_dot=True)
        if key not in self.parsed_content[section]:
            raise ValueError(
                f"Cannot delete {section}.{key} because {key} is not in {section}."
            )

        self.parsed_content.remove_option(section, key)
        return True

    def stringify(self) -> str:
        buffer = StringIO()
        self.parsed_content.write(buffer)
        return buffer.getvalue()

    def has(self, section_key: str) -> bool:
        if "." not in section_key:
            return self.parsed_content.has_section(section_key)

        section, key = split_on_dot(section_key, only_last_dot=True)
        return self.parsed_content.has_option(section, key)



import inspect
from pathlib import Path, PurePath
from shutil import copyfile

from config_file.parsers.base_parser import BaseParser
from config_file.parsers.ini_parser import IniParser
from config_file.utils import split_on_dot


class ConfigFile:
    def __init__(self, file_path, parser=None):
        """
        Saves the config file path and expands it if needed, reads in
        the file contents, and determines what parser should be used for
        the given file.

        :param file_path: The path to your configuration file.
        :param parser: A custom parser you'd like used for your config file.
                      It must be a concrete implementation of BaseParser.

        :raises ValueError: If the specified file path does not have an extension
                            that is supported or it is a directory.
        """
        self.path = self.__create_config_path(file_path)
        self.contents = self.__read_config_file()
        self.parser = self.__determine_parser(file_path, parser)

    @staticmethod
    def __create_config_path(file_path, original=False) -> str:
        if isinstance(file_path, PurePath):
            file_path = str(file_path)

        if len(file_path) > 1 and file_path[0] == "~":
            return str(Path(file_path).expanduser())

        if original:
            file_parts = split_on_dot(file_path, only_last_dot=True)
            file_parts.insert(-1, "original")
            file_path = ".".join(file_parts)

        if Path(file_path).is_dir():
            raise ValueError(f"The specified config file ({file_path}) is a directory.")

        return file_path

    def __determine_parser(self, file_path, parser):
        if isinstance(file_path, PurePath):
            file_path = str(file_path)

        if isinstance(parser, BaseParser) and not inspect.isabstract(parser):
            return parser(self.contents)

        file_type = split_on_dot(file_path, only_last_dot=True)[-1]
        if file_type == "ini":
            return IniParser(self.contents)
        else:
            raise ValueError(
                f"File path contains an unsupported or "
                f"unrecognized file type: {file_path}"
            )

    def __read_config_file(self):
        with open(self.path, "r") as file:
            return file.read()

    def get(self, key, parse_type=True):
        """
        Retrieve the value of a key in its native type.
        This means the string 'true' will be parsed back as the
        boolean True.

        If parse_type is set to False, all values will be returned
        back as strings.
        """
        return self.parser.get(key, parse_type=parse_type)

    def set(self, key, value):
        """Sets the value of a key."""
        return self.parser.set(key, value)

    def delete(self, section_key):
        """Deletes a key/value pair or entire sections."""
        return self.parser.delete(section_key)

    def stringify(self) -> str:
        """Convert the parsed internal representation of the file back into a string."""
        return self.parser.stringify()

    def has(self, section_key: str) -> bool:
        """
        Check if a section, sub-section, or key exists in the file
        using a section.key format.

        Some formats, like JSON, do not have sections and therefore,
        it would only be checking if the key exists.
        """
        return self.parser.has(section_key)

    def reset(self, original_file_path=None):
        """
        Resets the config file by deleting it and copying the original back
        in its place. The contents of this config file are then set to the
        newly reset config file.

        :param original_file_path: The path to the original config file to reset it to.
        If this is not provided and, say, your configuration file is config.json, the
        method would look for a config.original.json in the same folder by default.

        :return: True if the reset succeeded
        :raises OSError: If the original configuration file is not present or if an
                         error occurred when trying to copy the file.
        """
        if original_file_path is None:
            original_file_path = self.__create_config_path(
                str(self.path), original=True
            )

        if not Path(original_file_path).exists():
            raise OSError(f"The {original_file_path} file to reset to does not exist.")

        Path(self.path).expanduser().unlink(missing_ok=True)
        copyfile(original_file_path, self.path)

        self.contents = self.__read_config_file()
        self.parser.parsed_content = self.parser.parse(self.contents)

        return True

    def save(self):
        """
        Save the configuration changes by writing the file out to the specified path
        :return: True if the save succeeded.
        """
        with open(self.path, "w") as config_file:
            config_file.write(self.parser.stringify())

        return True
